import 'package:krow/core/entity/event_entity.dart';
import 'package:krow/core/sevices/create_event_service/data/models/create_event_input_model.dart';

abstract class CreateEventServiceRepository {
  Future<String> createEvent(CreateEventInputModel input);

  Future<void> deleteDraft(String id);
  Future<void> publishEvent(String id);

  Future<void> updateEvent(CreateEventInputModel input);


}
